@@ -18,19 +18,20 @@
 #  Last modified: 2021.11.15 at 14:35:51 CET
 from __future__ import annotations
 
-import stat
-
 import os
-import requests
 import shutil
+import stat
 import subprocess
+import re
 from pathlib import Path
 from typing import Callable, List, Tuple, Union
 
+import requests
+
 from .Exceptions import AlreadyGitRepository, DirtyRepository, InvalidUrl, NotGitRepository, PathNotFoundException
 
 
-class Repository:
+class Repository(object):
 
 	def __init__(self, directory: Union[str, Path], makeDir: bool = False, init: bool = False, url: str = '', raiseIfExisting: bool = False, quiet: bool = True):
 		if isinstance(directory, str):
@@ -64,6 +65,11 @@ class Repository:
 		branches       = self.execute('git branch')[0]
 		self.branches  = set(branches.split('\n'))
 
+	@property
+	def remote(self) -> dict[Remote]:
+		remotes = self.execute('git remote')[0]
+		return dict((rem.user, rem) for rem in [Remote(statusString=st) for st in remotes.split('\n')])
+
 
 	@classmethod
 	def clone(cls, url: str, directory: Union[str, Path], branch: str = 'master', makeDir: bool = False, force: bool = False, quiet: bool = True) -> Repository:
@@ -143,6 +149,11 @@ class Repository:
 		return status.isDirty()
 
 
+	def isUpToDate(self) -> bool:
+		status = self.status()
+		return status.isUpToDate()
+
+
 	def revert(self):
 		self.reset()
 		self.clean()
@@ -178,13 +189,7 @@ class Repository:
 		self.execute('git pull')
 
 
-	def fetch(self, force: bool = False):
-		if self.isDirty():
-			if not force:
-				raise DirtyRepository()
-			else:
-				self.revert()
-
+	def fetch(self):
 		self.execute('git fetch')
 
 
@@ -281,14 +286,41 @@ class Repository:
 		self._quiet = value
 
 
-class Status:
+class Status(object):
 
 	def __init__(self, directory: Union[str, Path]):
 		if isinstance(directory, str):
 			directory = Path(directory)
 
-		self._status = subprocess.run(f'git -C {str(directory)} status', capture_output=True, text=True, shell=True).stdout.strip()
+		self._directory = directory
 
 
 	def isDirty(self):
-		return 'working tree clean' not in self._status
+		status = subprocess.run(f'git -C {str(self._directory)} status', capture_output=True, text=True, shell=True).stdout.strip()
+		return 'working tree clean' not in status
+
+
+	def isUpToDate(self):
+		subprocess.run(f'git -C {str(self._directory)} fetch origin', shell=True)
+		status = subprocess.run(f'git -C {str(self._directory)} status', capture_output=True, text=True, shell=True).stdout.strip()
+		return 'Your branch is up to date with' in status
+
+class Remote(object):
+
+	def __init__(self, name: str = None, url: str = None, user: str = None, statusString: str = None):
+		"""
+		:param name:
+		:param url:
+		:param user: if not supplied is extracted from url/statusString
+		:param statusString: e.g. "origin  https://github.com/project-alice-assistant/skill_AliceCore.git (push)"
+		"""
+		self.name = name
+		self.url = url
+		self.user = user
+
+		if statusString is not None:
+			self.name, dummy, self.url, direction = statusString.split(' ')
+		if self.user is None:
+			match = re.search('github.com/(.+?)/', self.url)
+			if match:
+				self.user = match.group(1)
